using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Web.Hosting;
using System.Web;
using System.Reflection;
using System.IO;
using System.Web.Caching;
using System.Collections.Specialized;

namespace VirtualPathCommon
{
    public class AssemblyResourceVirtualPathProvider : System.Web.Hosting.VirtualPathProvider
    {
        /// <summary>
        /// AssemblyPathVirtualPathӳ
        /// </summary>
        private string _VirtualPath;
        private string _AssemblyPath;

        private string VirtualPath { get { return _VirtualPath; } }
        private string AssemblyPath { get { return _AssemblyPath; } }

        public AssemblyResourceVirtualPathProvider(string virtualPath, string assemblyPath)
        {
            _VirtualPath = "~/" + virtualPath;
            _AssemblyPath = assemblyPath;
        }

        /// <summary>
        /// ǷAssemblyResource͵VirtualPath
        /// </summary>
        /// <param name="virtualPath">virtualPath</param>
        /// <returns></returns>
        private bool IsAssemblyResourceVirtualPath(string virtualPath)
        {
            string path = VirtualPathUtility.ToAppRelative(virtualPath);
            return path.StartsWith(VirtualPath, StringComparison.InvariantCultureIgnoreCase);
        }

        /// <summary>
        /// ȡvirtualPathӦAssemblyڵResourceName
        /// </summary>
        /// <param name="virtualPath">virtualPath</param>
        /// <returns></returns>
        private string GetResourceName(string virtualPath)
        {
            return VirtualPathUtility.GetFileName(virtualPath);
        }

        #region override

        /// <summary>
        /// ȡVirtualFile
        /// </summary>
        /// <param name="virtualPath"></param>
        /// <returns></returns>
        public override VirtualFile GetFile(string virtualPath)
        {
            if (IsAssemblyResourceVirtualPath(virtualPath))
            {
                string resourceName = this.GetResourceName(virtualPath);
                return new AssemblyResourceVirtualFile(virtualPath, AssemblyPath, resourceName);
            }
            else
            {
                return Previous.GetFile(virtualPath);
            }
        }

        /// <summary>
        /// virtualPathָļǷڡ
        /// </summary>
        /// <param name="virtualPath"></param>
        /// <returns></returns>
        public override bool FileExists(string virtualPath)
        {
            if (IsAssemblyResourceVirtualPath(virtualPath))
            {
                //return true;
                Assembly assembly = Assembly.LoadFrom(AssemblyPath);
                if (assembly != null)
                {
                    string resourceName = this.GetResourceName(virtualPath);
                    return (assembly.GetManifestResourceInfo(resourceName) != null);
                }
                return false;
            }
            else
            {
                return Previous.FileExists(virtualPath);
            }
        }

        public override bool DirectoryExists(string virtualDir)
        {
            if (IsAssemblyResourceVirtualPath(virtualDir))
            {
                return true;
            }
            else
            {
                return Previous.DirectoryExists(virtualDir);
            }
        }

        public override string GetFileHash(string virtualPath, IEnumerable virtualPathDependencies)
        {
            return null;

            //HashCodeCombiner combiner = new HashCodeCombiner();
            //foreach (string str in virtualPathDependencies)
            //{
            //    string fileName = HostingEnvironment.MapPathInternal(str);
            //    combiner.AddFile(fileName);
            //}
            //return combiner.CombinedHashString;
        }

        public override CacheDependency GetCacheDependency(string virtualPath, IEnumerable virtualPathDependencies, DateTime utcStart)
        {
            return null;

            //System.Collections.Specialized.StringCollection fullPathDependencies = null;

            //// Get the full path to all dependencies.
            //foreach (string virtualDependency in virtualPathDependencies)
            //{
            //    if (fullPathDependencies == null)
            //        fullPathDependencies = new System.Collections.Specialized.StringCollection();

            //    fullPathDependencies.Add(virtualDependency);
            //}
            //if (fullPathDependencies == null)
            //    return null;

            //// Copy the list of full-path dependencies into an array.
            //string[] fullPathDependenciesArray = new string[fullPathDependencies.Count];
            //fullPathDependencies.CopyTo(fullPathDependenciesArray, 0);
            //// Copy the virtual path into an array.
            //string[] virtualPathArray = new string[1];
            //virtualPathArray[0] = virtualPath;

            //return new CacheDependency(virtualPathArray, fullPathDependenciesArray, utcStart);
        }

        //public override string CombineVirtualPaths(string basePath, string relativePath)
        //{
        //    if (IsAssemblyResourceVirtualPath(basePath))
        //    {
        //        return null;
        //    }
        //    else
        //    {
        //        return Previous.CombineVirtualPaths(basePath, relativePath);
        //    }
        //}

        //public override System.Runtime.Remoting.ObjRef CreateObjRef(Type requestedType)
        //{
        //    return Previous.CreateObjRef(requestedType);
        //}

        //public override string GetCacheKey(string virtualPath)
        //{
        //    if (IsAssemblyResourceVirtualPath(virtualPath))
        //    {
        //        return null;
        //    }
        //    else
        //    {
        //        return Previous.GetCacheKey(virtualPath);
        //    }
        //}
        #endregion
    }
}
